#include <add-ons/input_server/InputServerDevice.h>
#include "vmw_door.h"


class TVMWMouseDevice : public BInputServerDevice
{
public:
	TVMWMouseDevice() : BInputServerDevice()
	{
		fThreadID = -1;
		fPortID = -1;
		fLastpos = -1;
	};
	
	~TVMWMouseDevice()
	{
	};
	
	status_t InitCheck()
	{
		input_device_ref* devices[2];
		input_device_ref devRef = {"vmw_mouse_device", B_POINTING_DEVICE, (void *)this};
		devices[0] = &devRef;
		devices[1] = NULL;
		return RegisterDevices(devices);
	};
	
	status_t SystemShuttingDown()
	{
		return B_OK;
	};
	
	status_t Start(const char *device, void *cookie)
	{
		if (fThreadID < 0) {
			fPortID = create_port(50, "vmw_mouse_device_port");
			fThreadID = spawn_thread(TVMWMouseDevice::ThreadFunc,
									"vmw_mouse_device_thread",
									B_URGENT_DISPLAY_PRIORITY,
									(void*) this);
			resume_thread(fThreadID);
		}
		return B_OK;
	};
	
	status_t Stop(const char *device, void *cookie)
	{
		if (fThreadID > 0) {
			sem_id quitSem = create_sem(0, "vmw_mouse_device_quit_sem");
			write_port(fPortID, B_QUIT_REQUESTED, (void*) &quitSem, sizeof(quitSem));
			acquire_sem(quitSem);
			delete_sem(quitSem);
			delete_port(fPortID);
			fPortID = -1;
		}
		return B_OK;
	};
	
	status_t Control(const char *device, void *cookie, uint32 code, BMessage *msg)
	{
		return BInputServerDevice::Control(device, cookie, code, msg);
	};
	
private:	
	thread_id fThreadID;
	port_id fPortID;
	int fLastpos;
	
	void MessageLoop() {
		int32 what;
		int32 data;
		int p, x, y;
		BMessage *event;
		for (;;) {
			what = 0;
			data = 0;
			while (read_port_etc(fPortID, &what, &data, sizeof(data), B_TIMEOUT, 0) >= 0) {
				if (what == B_QUIT_REQUESTED) {
					release_sem((sem_id)data);
					return;
				}
			}
			p = vmw_get_mouse_pos();
/*			
			if (fLastpos != p) {
				x = p >> 16;
				y = p & 0xffff;
				event = new BMessage(B_MOUSE_MOVED);
				event->AddInt32("buttons", 0);
				event->AddPoint("where", BPoint(x, y));
				event->AddInt32("modifiers", 0);
				EnqueueMessage(event);
				fLastpos = p;
			}
*/			
			snooze(10000);
		}
	};
	
	static int32 ThreadFunc(void *castToTVMWMouseDevice) {
		TVMWMouseDevice *device = (TVMWMouseDevice *)castToTVMWMouseDevice;
		device->MessageLoop();
		return 0;
	};
};

extern "C" _EXPORT BInputServerDevice *instantiate_input_device()
{
	return new TVMWMouseDevice();
}
