#include <stdio.h>
#include <String.h>
#include "VMWServer.h"
#include "vmw_door.h"

int main()
{
	TVMWServer *app = new TVMWServer();
	app->Run();
	delete app;
	return 0;
}

TVMWServer::TVMWServer()
	:	BApplication("application/x-vnd.momoziro-vmw_server")
{
	fPastedFromHost = false;
	fMessageRunner = NULL;
	if (vmw_get_options() & VMW_OPT_ALLOW_COPY_PASTE) {
		BMessage msg('past');
		BMessenger target(be_app_messenger);
		fMessageRunner = new BMessageRunner(target, (const BMessage *)&msg, 100000);
		be_clipboard->StartWatching(this);
	} else {
		printf("vmw_server: Clipboard not supported.\n");
	}
}

TVMWServer::~TVMWServer()
{
	if (fMessageRunner) delete fMessageRunner;
	be_clipboard->StopWatching(this);
}

void TVMWServer::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case B_CLIPBOARD_CHANGED:
			ClipboardChanged(msg);
			break;
		case 'past':
			MessageRunner(msg);
			break;
		default: BApplication::MessageReceived(msg);
	}
}

void TVMWServer::ClipboardChanged(BMessage *msg)
{
	if (fPastedFromHost) {
		fPastedFromHost = false;
		return;
	}
		
	const char *text;
	int32 textLength;
	BMessage *clip;
	BString clipText;
	
	if (!be_clipboard->Lock()) return;
	clip = be_clipboard->Data();
	if (!clip) return;
	if (clip->FindData("text/plain", B_MIME_TYPE, (const void **)&text, &textLength) != B_OK) return;
	clipText.SetTo(text);
	be_clipboard->Unlock();
	clipText << '\0' << '\0' << '\0' << '\0';
	
	vmw_set_copy_text(textLength, clipText.String());
}

void TVMWServer::MessageRunner(BMessage *msg)
{
	uint32 pasteLength = vmw_get_paste_text_length();
	if (pasteLength == 0xffffffff) return;
	
	if (!be_clipboard->Lock()) return;
	
	fPastedFromHost = true;
	
	char textbuff[5] = "    ";
	uint32 *p = (uint32 *)textbuff;
	BString pasteText;
	while ((*p = vmw_get_paste_text()) != 0) {
		pasteText << textbuff;
	}
	
	be_clipboard->Clear();
	BMessage *clip = be_clipboard->Data();
	if (!clip) return;
	
	clip->AddData("text/plain", B_MIME_TYPE, pasteText.String(), pasteText.Length());
	be_clipboard->Commit();
	
	be_clipboard->Unlock();
}
